@@ -40,7 +40,7 @@ import shutil
 import subprocess
 from tempfile import mkdtemp, mkstemp
 from contextlib import contextmanager
-from StringIO import StringIO
+from six import StringIO
 from enum import Enum
 
 import beets
@@ -51,11 +51,12 @@ from beets.library import Library, Item, Album
 from beets import importer
 from beets.autotag.hooks import AlbumInfo, TrackInfo
 from beets.mediafile import MediaFile, Image
-from beets.ui import _arg_encoding
 from beets import util
+from beets.util import MoveOperation
 
 # TODO Move AutotagMock here
 from test import _common
+import six
 
 
 class LogCapture(logging.Handler):
@@ -65,7 +66,7 @@ class LogCapture(logging.Handler):
         self.messages = []
 
     def emit(self, record):
-        self.messages.append(unicode(record.msg))
+        self.messages.append(six.text_type(record.msg))
 
 
 @contextmanager
@@ -89,7 +90,8 @@ def control_stdin(input=None):
     """
     org = sys.stdin
     sys.stdin = StringIO(input)
-    sys.stdin.encoding = 'utf8'
+    if six.PY2:  # StringIO encoding attr isn't writable in python >= 3
+        sys.stdin.encoding = 'utf-8'
     try:
         yield sys.stdin
     finally:
@@ -108,7 +110,8 @@ def capture_stdout():
     """
     org = sys.stdout
     sys.stdout = capture = StringIO()
-    sys.stdout.encoding = 'utf8'
+    if six.PY2:  # StringIO encoding attr isn't writable in python >= 3
+        sys.stdout.encoding = 'utf-8'
     try:
         yield sys.stdout
     finally:
@@ -116,13 +119,25 @@ def capture_stdout():
         print(capture.getvalue())
 
 
+def _convert_args(args):
+    """Convert args to bytestrings for Python 2 and convert them to strings
+       on Python 3.
+    """
+    for i, elem in enumerate(args):
+        if six.PY2:
+            if isinstance(elem, six.text_type):
+                args[i] = elem.encode(util.arg_encoding())
+        else:
+            if isinstance(elem, bytes):
+                args[i] = elem.decode(util.arg_encoding())
+
+    return args
+
+
 def has_program(cmd, args=['--version']):
     """Returns `True` if `cmd` can be executed.
     """
-    full_cmd = [cmd] + args
-    for i, elem in enumerate(full_cmd):
-        if isinstance(elem, unicode):
-            full_cmd[i] = elem.encode(_arg_encoding())
+    full_cmd = _convert_args([cmd] + args)
     try:
         with open(os.devnull, 'wb') as devnull:
             subprocess.check_call(full_cmd, stderr=devnull,
@@ -166,7 +181,7 @@ class TestHelper(object):
         Make sure you call ``teardown_beets()`` afterwards.
         """
         self.create_temp_dir()
-        os.environ['BEETSDIR'] = self.temp_dir
+        os.environ['BEETSDIR'] = util.py3_path(self.temp_dir)
 
         self.config = beets.config
         self.config.clear()
@@ -179,10 +194,12 @@ class TestHelper(object):
 
         self.libdir = os.path.join(self.temp_dir, b'libdir')
         os.mkdir(self.libdir)
-        self.config['directory'] = self.libdir
+        self.config['directory'] = util.py3_path(self.libdir)
 
         if disk:
-            dbpath = self.config['library'].as_filename()
+            dbpath = util.bytestring_path(
+                self.config['library'].as_filename()
+            )
         else:
             dbpath = ':memory:'
         self.lib = Library(dbpath, self.libdir)
@@ -299,6 +316,8 @@ class TestHelper(object):
         item = Item(**values_)
         if 'path' not in values:
             item['path'] = 'audio.' + item['format'].lower()
+        # mtime needs to be set last since other assignments reset it.
+        item.mtime = 12345
         return item
 
     def add_item(self, **values):
@@ -331,7 +350,7 @@ class TestHelper(object):
         item['path'] = os.path.join(_common.RSRC,
                                     util.bytestring_path('min.' + extension))
         item.add(self.lib)
-        item.move(copy=True)
+        item.move(operation=MoveOperation.COPY)
         item.store()
         return item
 
@@ -349,8 +368,10 @@ class TestHelper(object):
             item = Item.from_path(path)
             item.album = u'\u00e4lbum {0}'.format(i)  # Check unicode paths
             item.title = u't\u00eftle {0}'.format(i)
+            # mtime needs to be set last since other assignments reset it.
+            item.mtime = 12345
             item.add(self.lib)
-            item.move(copy=True)
+            item.move(operation=MoveOperation.COPY)
             item.store()
             items.append(item)
         return items
@@ -361,11 +382,13 @@ class TestHelper(object):
         items = []
         path = os.path.join(_common.RSRC, util.bytestring_path('full.' + ext))
         for i in range(track_count):
-            item = Item.from_path(bytes(path))
+            item = Item.from_path(path)
             item.album = u'\u00e4lbum'  # Check unicode paths
             item.title = u't\u00eftle {0}'.format(i)
+            # mtime needs to be set last since other assignments reset it.
+            item.mtime = 12345
             item.add(self.lib)
-            item.move(copy=True)
+            item.move(operation=MoveOperation.COPY)
             item.store()
             items.append(item)
         return self.lib.add_album(items)
@@ -416,17 +439,22 @@ class TestHelper(object):
 
     # Running beets commands
 
-    def run_command(self, *args):
+    def run_command(self, *args, **kwargs):
+        """Run a beets command with an arbitrary amount of arguments. The
+           Library` defaults to `self.lib`, but can be overridden with
+           the keyword argument `lib`.
+        """
+        sys.argv = ['beet']  # avoid leakage from test suite args
+        lib = None
         if hasattr(self, 'lib'):
             lib = self.lib
-        else:
-            lib = Library(':memory:')
-        beets.ui._raw_main(list(args), lib)
+        lib = kwargs.get('lib', lib)
+        beets.ui._raw_main(_convert_args(list(args)), lib)
 
     def run_with_output(self, *args):
         with capture_stdout() as out:
             self.run_command(*args)
-        return out.getvalue().decode('utf-8')
+        return util.text_string(out.getvalue())
 
     # Safe file operations
 
@@ -434,7 +462,8 @@ class TestHelper(object):
         """Create a temporary directory and assign it into
         `self.temp_dir`. Call `remove_temp_dir` later to delete it.
         """
-        self.temp_dir = mkdtemp()
+        temp_dir = mkdtemp()
+        self.temp_dir = util.bytestring_path(temp_dir)
 
     def remove_temp_dir(self):
         """Delete the temporary directory created by `create_temp_dir`.
@@ -506,7 +535,7 @@ class TestImportSession(importer.ImportSession):
 
     choose_item = choose_match
 
-    Resolution = Enum('Resolution', 'REMOVE SKIP KEEPBOTH')
+    Resolution = Enum('Resolution', 'REMOVE SKIP KEEPBOTH MERGE')
 
     default_resolution = 'REMOVE'
 
@@ -524,6 +553,8 @@ class TestImportSession(importer.ImportSession):
             task.set_choice(importer.action.SKIP)
         elif res == self.Resolution.REMOVE:
             task.should_remove_duplicates = True
+        elif res == self.Resolution.MERGE:
+            task.should_merge_duplicates = True
 
 
 def generate_album_info(album_id, track_ids):
