@@ -19,9 +19,9 @@ from __future__ import division, absolute_import, print_function
 
 import re
 import copy
+import unittest
 
 from test import _common
-from test._common import unittest
 from beets import autotag
 from beets.autotag import match
 from beets.autotag.hooks import Distance, string_dist
@@ -611,7 +611,7 @@ class AssignmentTest(unittest.TestCase):
             match.assign_items(items, trackinfo)
         self.assertEqual(extra_items, [])
         self.assertEqual(extra_tracks, [])
-        for item, info in mapping.iteritems():
+        for item, info in mapping.items():
             self.assertEqual(items.index(item), trackinfo.index(info))
 
 
