@@ -15,16 +15,26 @@
 
 from __future__ import division, absolute_import, print_function
 
+import sys
 import re
 import os.path
+import unittest
+
 from test import _common
-from test._common import unittest
 from test import helper
-from test.helper import control_stdin
+from test.helper import control_stdin, capture_log
 
 from beets.mediafile import MediaFile
 from beets import util
-from beets import ui
+
+
+def shell_quote(text):
+    if sys.version_info[0] < 3:
+        import pipes
+        return pipes.quote(text)
+    else:
+        import shlex
+        return shlex.quote(text)
 
 
 class TestHelper(helper.TestHelper):
@@ -39,31 +49,41 @@ class TestHelper(helper.TestHelper):
 
         # A Python script that copies the file and appends a tag.
         stub = os.path.join(_common.RSRC, b'convert_stub.py').decode('utf-8')
-        return u"python '{}' $source $dest {}".format(stub, tag)
+        return u"{} {} $source $dest {}".format(shell_quote(sys.executable),
+                                                shell_quote(stub), tag)
 
     def assertFileTag(self, path, tag):  # noqa
         """Assert that the path is a file and the files content ends with `tag`.
         """
+        display_tag = tag
         tag = tag.encode('utf-8')
         self.assertTrue(os.path.isfile(path),
-                        u'{0} is not a file'.format(path))
+                        u'{0} is not a file'.format(
+                            util.displayable_path(path)))
         with open(path, 'rb') as f:
-            f.seek(-len(tag), os.SEEK_END)
+            f.seek(-len(display_tag), os.SEEK_END)
             self.assertEqual(f.read(), tag,
-                             u'{0} is not tagged with {1}'.format(path, tag))
+                             u'{0} is not tagged with {1}'
+                             .format(
+                                 util.displayable_path(path),
+                                 display_tag))
 
     def assertNoFileTag(self, path, tag):  # noqa
         """Assert that the path is a file and the files content does not
         end with `tag`.
         """
+        display_tag = tag
         tag = tag.encode('utf-8')
         self.assertTrue(os.path.isfile(path),
-                        u'{0} is not a file'.format(path))
+                        u'{0} is not a file'.format(
+                            util.displayable_path(path)))
         with open(path, 'rb') as f:
             f.seek(-len(tag), os.SEEK_END)
             self.assertNotEqual(f.read(), tag,
                                 u'{0} is unexpectedly tagged with {1}'
-                                .format(path, tag))
+                                .format(
+                                    util.displayable_path(path),
+                                    display_tag))
 
 
 @_common.slow_test()
@@ -110,7 +130,7 @@ class ConvertCommand(object):
         """Run the `convert` command on a given path."""
         # The path is currently a filesystem bytestring. Convert it to
         # an argument bytestring.
-        path = path.decode(util._fsencoding()).encode(ui._arg_encoding())
+        path = path.decode(util._fsencoding()).encode(util.arg_encoding())
 
         args = args + (b'path:' + path,)
         return self.run_command('convert', *args)
@@ -160,7 +180,7 @@ class ConvertCliTest(unittest.TestCase, TestHelper, ConvertCommand):
         converted = os.path.join(self.convert_dest, b'converted.mp3')
         self.assertFileTag(converted, 'mp3')
 
-    def test_rejecet_confirmation(self):
+    def test_reject_confirmation(self):
         with control_stdin('n'):
             self.run_convert()
         converted = os.path.join(self.convert_dest, b'converted.mp3')
@@ -207,6 +227,11 @@ class ConvertCliTest(unittest.TestCase, TestHelper, ConvertCommand):
         converted = os.path.join(self.convert_dest, b'converted.mp3')
         self.assertFalse(os.path.exists(converted))
 
+    def test_empty_query(self):
+        with capture_log('beets.convert') as logs:
+            self.run_convert('An impossible query')
+        self.assertEqual(logs[0], u'convert: Empty query result.')
+
 
 @_common.slow_test()
 class NeverConvertLossyFilesTest(unittest.TestCase, TestHelper,
@@ -259,5 +284,6 @@ class NeverConvertLossyFilesTest(unittest.TestCase, TestHelper,
 def suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
 
+
 if __name__ == '__main__':
     unittest.main(defaultTest='suite')
