@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
 # Copyright 2016, Adrian Sampson and Diego Moreda.
 #
@@ -14,13 +15,14 @@
 
 from __future__ import division, absolute_import, print_function
 import codecs
+import unittest
 
 from mock import patch
 from test import _common
-from test._common import unittest
 from test.helper import TestHelper, control_stdin
 from test.test_ui_importer import TerminalImportSessionSetup
 from test.test_importer import ImportHelper, AutotagStub
+from beets.dbcore.query import TrueQuery
 from beets.library import Item
 from beetsplug.edit import EditPlugin
 
@@ -54,18 +56,18 @@ class ModifyFileMocker(object):
         `self.contents` is empty, the file remains unchanged.
         """
         if self.contents:
-            with codecs.open(filename, 'w', encoding='utf8') as f:
+            with codecs.open(filename, 'w', encoding='utf-8') as f:
                 f.write(self.contents)
 
     def replace_contents(self, filename, log):
         """Modify `filename`, reading its contents and replacing the strings
         specified in `self.replacements`.
         """
-        with codecs.open(filename, 'r', encoding='utf8') as f:
+        with codecs.open(filename, 'r', encoding='utf-8') as f:
             contents = f.read()
-        for old, new_ in self.replacements.iteritems():
+        for old, new_ in self.replacements.items():
             contents = contents.replace(old, new_)
-        with codecs.open(filename, 'w', encoding='utf8') as f:
+        with codecs.open(filename, 'w', encoding='utf-8') as f:
             f.write(contents)
 
 
@@ -106,6 +108,7 @@ class EditMixin(object):
 
 
 @_common.slow_test()
+@patch('beets.library.Item.write')
 class EditCommandTest(unittest.TestCase, TestHelper, EditMixin):
     """Black box tests for `beetsplug.edit`. Command line interaction is
     simulated using `test.helper.control_stdin()`, and yaml editing via an
@@ -123,26 +126,21 @@ class EditCommandTest(unittest.TestCase, TestHelper, EditMixin):
         self.items_orig = [{f: item[f] for f in item._fields} for
                            item in self.album.items()]
 
-        # Keep track of write()s.
-        self.write_patcher = patch('beets.library.Item.write')
-        self.mock_write = self.write_patcher.start()
-
     def tearDown(self):
         EditPlugin.listeners = None
-        self.write_patcher.stop()
         self.teardown_beets()
         self.unload_plugins()
 
-    def assertCounts(self, album_count=ALBUM_COUNT, track_count=TRACK_COUNT,  # noqa
+    def assertCounts(self, mock_write, album_count=ALBUM_COUNT, track_count=TRACK_COUNT,  # noqa
                      write_call_count=TRACK_COUNT, title_starts_with=''):
         """Several common assertions on Album, Track and call counts."""
         self.assertEqual(len(self.lib.albums()), album_count)
         self.assertEqual(len(self.lib.items()), track_count)
-        self.assertEqual(self.mock_write.call_count, write_call_count)
+        self.assertEqual(mock_write.call_count, write_call_count)
         self.assertTrue(all(i.title.startswith(title_starts_with)
                             for i in self.lib.items()))
 
-    def test_title_edit_discard(self):
+    def test_title_edit_discard(self, mock_write):
         """Edit title for all items in the library, then discard changes."""
         # Edit track titles.
         self.run_mocked_command({'replacements': {u't\u00eftle':
@@ -150,11 +148,11 @@ class EditCommandTest(unittest.TestCase, TestHelper, EditMixin):
                                 # Cancel.
                                 ['c'])
 
-        self.assertCounts(write_call_count=0,
+        self.assertCounts(mock_write, write_call_count=0,
                           title_starts_with=u't\u00eftle')
         self.assertItemFieldsModified(self.album.items(), self.items_orig, [])
 
-    def test_title_edit_apply(self):
+    def test_title_edit_apply(self, mock_write):
         """Edit title for all items in the library, then apply changes."""
         # Edit track titles.
         self.run_mocked_command({'replacements': {u't\u00eftle':
@@ -162,12 +160,12 @@ class EditCommandTest(unittest.TestCase, TestHelper, EditMixin):
                                 # Apply changes.
                                 ['a'])
 
-        self.assertCounts(write_call_count=self.TRACK_COUNT,
+        self.assertCounts(mock_write, write_call_count=self.TRACK_COUNT,
                           title_starts_with=u'modified t\u00eftle')
         self.assertItemFieldsModified(self.album.items(), self.items_orig,
-                                      ['title'])
+                                      ['title', 'mtime'])
 
-    def test_single_title_edit_apply(self):
+    def test_single_title_edit_apply(self, mock_write):
         """Edit title for one item in the library, then apply changes."""
         # Edit one track title.
         self.run_mocked_command({'replacements': {u't\u00eftle 9':
@@ -175,25 +173,25 @@ class EditCommandTest(unittest.TestCase, TestHelper, EditMixin):
                                 # Apply changes.
                                 ['a'])
 
-        self.assertCounts(write_call_count=1,)
+        self.assertCounts(mock_write, write_call_count=1,)
         # No changes except on last item.
         self.assertItemFieldsModified(list(self.album.items())[:-1],
                                       self.items_orig[:-1], [])
         self.assertEqual(list(self.album.items())[-1].title,
                          u'modified t\u00eftle 9')
 
-    def test_noedit(self):
+    def test_noedit(self, mock_write):
         """Do not edit anything."""
         # Do not edit anything.
         self.run_mocked_command({'contents': None},
                                 # No stdin.
                                 [])
 
-        self.assertCounts(write_call_count=0,
+        self.assertCounts(mock_write, write_call_count=0,
                           title_starts_with=u't\u00eftle')
         self.assertItemFieldsModified(self.album.items(), self.items_orig, [])
 
-    def test_album_edit_apply(self):
+    def test_album_edit_apply(self, mock_write):
         """Edit the album field for all items in the library, apply changes.
         By design, the album should not be updated.""
         """
@@ -203,27 +201,31 @@ class EditCommandTest(unittest.TestCase, TestHelper, EditMixin):
                                 # Apply changes.
                                 ['a'])
 
-        self.assertCounts(write_call_count=self.TRACK_COUNT)
+        self.assertCounts(mock_write, write_call_count=self.TRACK_COUNT)
         self.assertItemFieldsModified(self.album.items(), self.items_orig,
-                                      ['album'])
+                                      ['album', 'mtime'])
         # Ensure album is *not* modified.
         self.album.load()
         self.assertEqual(self.album.album, u'\u00e4lbum')
 
-    def test_single_edit_add_field(self):
+    def test_single_edit_add_field(self, mock_write):
         """Edit the yaml file appending an extra field to the first item, then
         apply changes."""
-        # Append "foo: bar" to item with id == 1.
-        self.run_mocked_command({'replacements': {u"id: 1":
-                                                  u"id: 1\nfoo: bar"}},
+        # Append "foo: bar" to item with id == 2. ("id: 1" would match both
+        # "id: 1" and "id: 10")
+        self.run_mocked_command({'replacements': {u"id: 2":
+                                                  u"id: 2\nfoo: bar"}},
                                 # Apply changes.
                                 ['a'])
 
-        self.assertEqual(self.lib.items(u'id:1')[0].foo, 'bar')
-        self.assertCounts(write_call_count=1,
+        self.assertEqual(self.lib.items(u'id:2')[0].foo, 'bar')
+        # Even though a flexible attribute was written (which is not directly
+        # written to the tags), write should still be called since templates
+        # might use it.
+        self.assertCounts(mock_write, write_call_count=1,
                           title_starts_with=u't\u00eftle')
 
-    def test_a_album_edit_apply(self):
+    def test_a_album_edit_apply(self, mock_write):
         """Album query (-a), edit album field, apply changes."""
         self.run_mocked_command({'replacements': {u'\u00e4lbum':
                                                   u'modified \u00e4lbum'}},
@@ -232,12 +234,12 @@ class EditCommandTest(unittest.TestCase, TestHelper, EditMixin):
                                 args=['-a'])
 
         self.album.load()
-        self.assertCounts(write_call_count=self.TRACK_COUNT)
+        self.assertCounts(mock_write, write_call_count=self.TRACK_COUNT)
         self.assertEqual(self.album.album, u'modified \u00e4lbum')
         self.assertItemFieldsModified(self.album.items(), self.items_orig,
-                                      ['album'])
+                                      ['album', 'mtime'])
 
-    def test_a_albumartist_edit_apply(self):
+    def test_a_albumartist_edit_apply(self, mock_write):
         """Album query (-a), edit albumartist field, apply changes."""
         self.run_mocked_command({'replacements': {u'album artist':
                                                   u'modified album artist'}},
@@ -246,12 +248,12 @@ class EditCommandTest(unittest.TestCase, TestHelper, EditMixin):
                                 args=['-a'])
 
         self.album.load()
-        self.assertCounts(write_call_count=self.TRACK_COUNT)
+        self.assertCounts(mock_write, write_call_count=self.TRACK_COUNT)
         self.assertEqual(self.album.albumartist, u'the modified album artist')
         self.assertItemFieldsModified(self.album.items(), self.items_orig,
-                                      ['albumartist'])
+                                      ['albumartist', 'mtime'])
 
-    def test_malformed_yaml(self):
+    def test_malformed_yaml(self, mock_write):
         """Edit the yaml file incorrectly (resulting in a malformed yaml
         document)."""
         # Edit the yaml file to an invalid file.
@@ -259,10 +261,10 @@ class EditCommandTest(unittest.TestCase, TestHelper, EditMixin):
                                 # Edit again to fix? No.
                                 ['n'])
 
-        self.assertCounts(write_call_count=0,
+        self.assertCounts(mock_write, write_call_count=0,
                           title_starts_with=u't\u00eftle')
 
-    def test_invalid_yaml(self):
+    def test_invalid_yaml(self, mock_write):
         """Edit the yaml file incorrectly (resulting in a well-formed but
         invalid yaml document)."""
         # Edit the yaml file to an invalid but parseable file.
@@ -270,7 +272,7 @@ class EditCommandTest(unittest.TestCase, TestHelper, EditMixin):
                                 # No stdin.
                                 [])
 
-        self.assertCounts(write_call_count=0,
+        self.assertCounts(mock_write, write_call_count=0,
                           title_starts_with=u't\u00eftle')
 
 
@@ -362,6 +364,34 @@ class EditDuringImporterTest(TerminalImportSessionSetup, unittest.TestCase,
         # Ensure album is fetched from a candidate.
         self.assertIn('albumid', self.lib.albums()[0].mb_albumid)
 
+    def test_edit_retag_apply(self):
+        """Import the album using a candidate, then retag and edit and apply
+        changes.
+        """
+        self._setup_import_session()
+        self.run_mocked_interpreter({},
+                                    # 1, Apply changes.
+                                    ['1', 'a'])
+
+        # Retag and edit track titles.  On retag, the importer will reset items
+        # ids but not the db connections.
+        self.importer.paths = []
+        self.importer.query = TrueQuery()
+        self.run_mocked_interpreter({'replacements': {u'Applied Title':
+                                                      u'Edited Title'}},
+                                    # eDit, Apply changes.
+                                    ['d', 'a'])
+
+        # Check that 'title' field is modified, and other fields come from
+        # the candidate.
+        self.assertTrue(all('Edited Title ' in i.title
+                            for i in self.lib.items()))
+        self.assertTrue(all('match ' in i.mb_trackid
+                            for i in self.lib.items()))
+
+        # Ensure album is fetched from a candidate.
+        self.assertIn('albumid', self.lib.albums()[0].mb_albumid)
+
     def test_edit_discard_candidate(self):
         """Edit the album field for all items in the library, discard changes,
         using a candidate.
