@@ -20,17 +20,18 @@ from __future__ import division, absolute_import, print_function
 import os
 import re
 import shutil
-import StringIO
 import unicodedata
 import sys
+import stat
+from six import StringIO
 from tempfile import mkstemp
 from zipfile import ZipFile
 from tarfile import TarFile
-from mock import patch
+from mock import patch, Mock
+import unittest
 
 from test import _common
-from test._common import unittest
-from beets.util import displayable_path, bytestring_path
+from beets.util import displayable_path, bytestring_path, py3_path
 from test.helper import TestImportSession, TestHelper, has_program, capture_log
 from beets import importer
 from beets.importer import albums_in_dir
@@ -209,7 +210,8 @@ class ImportHelper(TestHelper):
 
     def _setup_import_session(self, import_dir=None, delete=False,
                               threaded=False, copy=True, singletons=False,
-                              move=False, autotag=True, link=False):
+                              move=False, autotag=True, link=False,
+                              hardlink=False):
         config['import']['copy'] = copy
         config['import']['delete'] = delete
         config['import']['timid'] = True
@@ -219,6 +221,7 @@ class ImportHelper(TestHelper):
         config['import']['autotag'] = autotag
         config['import']['resume'] = False
         config['import']['link'] = link
+        config['import']['hardlink'] = hardlink
 
         self.importer = TestImportSession(
             self.lib, loghandler=None, query=None,
@@ -348,16 +351,38 @@ class NonAutotaggedImportTest(_common.TestCase, ImportHelper):
             )
             self.assertExists(filename)
             self.assertTrue(os.path.islink(filename))
-            self.assert_equal_path(os.readlink(filename), mediafile.path)
+            self.assert_equal_path(
+                util.bytestring_path(os.readlink(filename)),
+                mediafile.path
+            )
+
+    @unittest.skipUnless(_common.HAVE_HARDLINK, "need hardlinks")
+    def test_import_hardlink_arrives(self):
+        config['import']['hardlink'] = True
+        self.importer.run()
+        for mediafile in self.import_media:
+            filename = os.path.join(
+                self.libdir,
+                b'Tag Artist', b'Tag Album',
+                util.bytestring_path('{0}.mp3'.format(mediafile.title))
+            )
+            self.assertExists(filename)
+            s1 = os.stat(mediafile.path)
+            s2 = os.stat(filename)
+            self.assertTrue(
+                (s1[stat.ST_INO], s1[stat.ST_DEV]) ==
+                (s2[stat.ST_INO], s2[stat.ST_DEV])
+            )
 
 
 def create_archive(session):
-    (handle, path) = mkstemp(dir=session.temp_dir)
+    (handle, path) = mkstemp(dir=py3_path(session.temp_dir))
     os.close(handle)
-    archive = ZipFile(path, mode='w')
-    archive.write(os.path.join(_common.RSRC, 'full.mp3'),
+    archive = ZipFile(py3_path(path), mode='w')
+    archive.write(os.path.join(_common.RSRC, b'full.mp3'),
                   'full.mp3')
     archive.close()
+    path = bytestring_path(path)
     return path
 
 
@@ -410,8 +435,8 @@ class ImportTarTest(ImportZipTest):
     def create_archive(self):
         (handle, path) = mkstemp(dir=self.temp_dir)
         os.close(handle)
-        archive = TarFile(path, mode='w')
-        archive.add(os.path.join(_common.RSRC, 'full.mp3'),
+        archive = TarFile(py3_path(path), mode='w')
+        archive.add(os.path.join(_common.RSRC, b'full.mp3'),
                     'full.mp3')
         archive.close()
         return path
@@ -421,14 +446,14 @@ class ImportTarTest(ImportZipTest):
 class ImportRarTest(ImportZipTest):
 
     def create_archive(self):
-        return os.path.join(_common.RSRC, 'archive.rar')
+        return os.path.join(_common.RSRC, b'archive.rar')
 
 
 @unittest.skip('Implement me!')
 class ImportPasswordRarTest(ImportZipTest):
 
     def create_archive(self):
-        return os.path.join(_common.RSRC, 'password.rar')
+        return os.path.join(_common.RSRC, b'password.rar')
 
 
 class ImportSingletonTest(_common.TestCase, ImportHelper):
@@ -518,6 +543,38 @@ class ImportSingletonTest(_common.TestCase, ImportHelper):
         self.assertEqual(len(self.lib.items()), 2)
         self.assertEqual(len(self.lib.albums()), 2)
 
+    def test_set_fields(self):
+        genre = u"\U0001F3B7 Jazz"
+        collection = u"To Listen"
+
+        config['import']['set_fields'] = {
+            u'collection': collection,
+            u'genre': genre
+        }
+
+        # As-is item import.
+        self.assertEqual(self.lib.albums().get(), None)
+        self.importer.add_choice(importer.action.ASIS)
+        self.importer.run()
+
+        for item in self.lib.items():
+            item.load()  # TODO: Not sure this is necessary.
+            self.assertEqual(item.genre, genre)
+            self.assertEqual(item.collection, collection)
+            # Remove item from library to test again with APPLY choice.
+            item.remove()
+
+        # Autotagged.
+        self.assertEqual(self.lib.albums().get(), None)
+        self.importer.clear_choices()
+        self.importer.add_choice(importer.action.APPLY)
+        self.importer.run()
+
+        for item in self.lib.items():
+            item.load()
+            self.assertEqual(item.genre, genre)
+            self.assertEqual(item.collection, collection)
+
 
 class ImportTest(_common.TestCase, ImportHelper):
     """Test APPLY, ASIS and SKIP choices.
@@ -576,6 +633,17 @@ class ImportTest(_common.TestCase, ImportHelper):
         self.assert_file_in_lib(
             b'Applied Artist', b'Applied Album', b'Applied Title 1.mp3')
 
+    def test_apply_from_scratch_removes_other_metadata(self):
+        config['import']['from_scratch'] = True
+
+        for mediafile in self.import_media:
+            mediafile.genre = u'Tag Genre'
+            mediafile.save()
+
+        self.importer.add_choice(importer.action.APPLY)
+        self.importer.run()
+        self.assertEqual(self.lib.items().get().genre, u'')
+
     def test_apply_with_move_deletes_import(self):
         config['import']['move'] = True
 
@@ -647,6 +715,38 @@ class ImportTest(_common.TestCase, ImportHelper):
         with self.assertRaises(AttributeError):
             self.lib.items().get().data_source
 
+    def test_set_fields(self):
+        genre = u"\U0001F3B7 Jazz"
+        collection = u"To Listen"
+
+        config['import']['set_fields'] = {
+            u'collection': collection,
+            u'genre': genre
+        }
+
+        # As-is album import.
+        self.assertEqual(self.lib.albums().get(), None)
+        self.importer.add_choice(importer.action.ASIS)
+        self.importer.run()
+
+        for album in self.lib.albums():
+            album.load()  # TODO: Not sure this is necessary.
+            self.assertEqual(album.genre, genre)
+            self.assertEqual(album.collection, collection)
+            # Remove album from library to test again with APPLY choice.
+            album.remove()
+
+        # Autotagged.
+        self.assertEqual(self.lib.albums().get(), None)
+        self.importer.clear_choices()
+        self.importer.add_choice(importer.action.APPLY)
+        self.importer.run()
+
+        for album in self.lib.albums():
+            album.load()
+            self.assertEqual(album.genre, genre)
+            self.assertEqual(album.collection, collection)
+
 
 class ImportTracksTest(_common.TestCase, ImportHelper):
     """Test TRACKS and APPLY choice.
@@ -1068,7 +1168,7 @@ class InferAlbumDataTest(_common.TestCase):
         self.assertFalse(self.items[0].comp)
 
 
-def test_album_info():
+def test_album_info(*args, **kwargs):
     """Create an AlbumInfo object for testing.
     """
     track_info = TrackInfo(
@@ -1083,9 +1183,10 @@ def test_album_info():
         album_id=u'albumid',
         artist_id=u'artistid',
     )
-    return album_info
+    return iter([album_info])
 
 
+@patch('beets.autotag.mb.match_album', Mock(side_effect=test_album_info))
 class ImportDuplicateAlbumTest(unittest.TestCase, TestHelper,
                                _common.Assertions):
 
@@ -1095,17 +1196,11 @@ class ImportDuplicateAlbumTest(unittest.TestCase, TestHelper,
         # Original album
         self.add_album_fixture(albumartist=u'artist', album=u'album')
 
-        # Create duplicate through autotagger
-        self.match_album_patcher = patch('beets.autotag.mb.match_album')
-        self.match_album = self.match_album_patcher.start()
-        self.match_album.return_value = iter([test_album_info()])
-
         # Create import session
         self.importer = self.create_importer()
         config['import']['autotag'] = True
 
     def tearDown(self):
-        self.match_album_patcher.stop()
         self.teardown_beets()
 
     def test_remove_duplicate_album(self):
@@ -1164,6 +1259,12 @@ class ImportDuplicateAlbumTest(unittest.TestCase, TestHelper,
         item = self.lib.items().get()
         self.assertEqual(item.title, u't\xeftle 0')
 
+    def test_merge_duplicate_album(self):
+        self.importer.default_resolution = self.importer.Resolution.MERGE
+        self.importer.run()
+
+        self.assertEqual(len(self.lib.albums()), 1)
+
     def test_twice_in_import_dir(self):
         self.skipTest('write me')
 
@@ -1175,6 +1276,13 @@ class ImportDuplicateAlbumTest(unittest.TestCase, TestHelper,
         return album
 
 
+def test_track_info(*args, **kwargs):
+    return iter([TrackInfo(
+        artist=u'artist', title=u'title',
+        track_id=u'new trackid', index=0,)])
+
+
+@patch('beets.autotag.mb.match_track', Mock(side_effect=test_track_info))
 class ImportDuplicateSingletonTest(unittest.TestCase, TestHelper,
                                    _common.Assertions):
 
@@ -1185,24 +1293,12 @@ class ImportDuplicateSingletonTest(unittest.TestCase, TestHelper,
         self.add_item_fixture(artist=u'artist', title=u'title',
                               mb_trackid='old trackid')
 
-        # Create duplicate through autotagger
-        self.match_track_patcher = patch('beets.autotag.mb.match_track')
-        self.match_track = self.match_track_patcher.start()
-        track_info = TrackInfo(
-            artist=u'artist',
-            title=u'title',
-            track_id=u'new trackid',
-            index=0,
-        )
-        self.match_track.return_value = iter([track_info])
-
         # Import session
         self.importer = self.create_importer()
         config['import']['autotag'] = True
         config['import']['singletons'] = True
 
     def tearDown(self):
-        self.match_track_patcher.stop()
         self.teardown_beets()
 
     def test_remove_duplicate(self):
@@ -1250,14 +1346,14 @@ class ImportDuplicateSingletonTest(unittest.TestCase, TestHelper,
 
 class TagLogTest(_common.TestCase):
     def test_tag_log_line(self):
-        sio = StringIO.StringIO()
+        sio = StringIO()
         handler = logging.StreamHandler(sio)
         session = _common.import_session(loghandler=handler)
         session.tag_log('status', 'path')
         self.assertIn('status path', sio.getvalue())
 
     def test_tag_log_unicode(self):
-        sio = StringIO.StringIO()
+        sio = StringIO()
         handler = logging.StreamHandler(sio)
         session = _common.import_session(loghandler=handler)
         session.tag_log('status', u'caf\xe9')  # send unicode
@@ -1471,10 +1567,10 @@ class MultiDiscAlbumsInDirTest(_common.TestCase):
         """Normalize a path's Unicode combining form according to the
         platform.
         """
-        path = path.decode('utf8')
+        path = path.decode('utf-8')
         norm_form = 'NFD' if sys.platform == 'darwin' else 'NFC'
         path = unicodedata.normalize(norm_form, path)
-        return path.encode('utf8')
+        return path.encode('utf-8')
 
     def test_coalesce_nested_album_multiple_subdirs(self):
         self.create_music()
@@ -1702,7 +1798,81 @@ class ImportPretendTest(_common.TestCase, ImportHelper):
         self.assertEqual(logs, [u'No files imported from {0}'
                          .format(displayable_path(self.empty_path))])
 
+# Helpers for ImportMusicBrainzIdTest.
 
+
+def mocked_get_release_by_id(id_, includes=[], release_status=[],
+                             release_type=[]):
+    """Mimic musicbrainzngs.get_release_by_id, accepting only a restricted list
+    of MB ids (ID_RELEASE_0, ID_RELEASE_1). The returned dict differs only in
+    the release title and artist name, so that ID_RELEASE_0 is a closer match
+    to the items created by ImportHelper._create_import_dir()."""
+    # Map IDs to (release title, artist), so the distances are different.
+    releases = {ImportMusicBrainzIdTest.ID_RELEASE_0: ('VALID_RELEASE_0',
+                                                       'TAG ARTIST'),
+                ImportMusicBrainzIdTest.ID_RELEASE_1: ('VALID_RELEASE_1',
+                                                       'DISTANT_MATCH')}
+
+    return {
+        'release': {
+            'title': releases[id_][0],
+            'id': id_,
+            'medium-list': [{
+                'track-list': [{
+                    'recording': {
+                        'title': 'foo',
+                        'id': 'bar',
+                        'length': 59,
+                    },
+                    'position': 9,
+                    'number': 'A2'
+                }],
+                'position': 5,
+            }],
+            'artist-credit': [{
+                'artist': {
+                    'name': releases[id_][1],
+                    'id': 'some-id',
+                },
+            }],
+            'release-group': {
+                'id': 'another-id',
+            }
+        }
+    }
+
+
+def mocked_get_recording_by_id(id_, includes=[], release_status=[],
+                               release_type=[]):
+    """Mimic musicbrainzngs.get_recording_by_id, accepting only a restricted
+    list of MB ids (ID_RECORDING_0, ID_RECORDING_1). The returned dict differs
+    only in the recording title and artist name, so that ID_RECORDING_0 is a
+    closer match to the items created by ImportHelper._create_import_dir()."""
+    # Map IDs to (recording title, artist), so the distances are different.
+    releases = {ImportMusicBrainzIdTest.ID_RECORDING_0: ('VALID_RECORDING_0',
+                                                         'TAG ARTIST'),
+                ImportMusicBrainzIdTest.ID_RECORDING_1: ('VALID_RECORDING_1',
+                                                         'DISTANT_MATCH')}
+
+    return {
+        'recording': {
+            'title': releases[id_][0],
+            'id': id_,
+            'length': 59,
+            'artist-credit': [{
+                'artist': {
+                    'name': releases[id_][1],
+                    'id': 'some-id',
+                },
+            }],
+        }
+    }
+
+
+@patch('musicbrainzngs.get_recording_by_id',
+       Mock(side_effect=mocked_get_recording_by_id))
+@patch('musicbrainzngs.get_release_by_id',
+       Mock(side_effect=mocked_get_release_by_id))
 class ImportMusicBrainzIdTest(_common.TestCase, ImportHelper):
     """Test the --musicbrainzid argument."""
 
@@ -1717,17 +1887,7 @@ class ImportMusicBrainzIdTest(_common.TestCase, ImportHelper):
         self.setup_beets()
         self._create_import_dir(1)
 
-        # Patch calls to musicbrainzngs.
-        self.release_patcher = patch('musicbrainzngs.get_release_by_id',
-                                     side_effect=mocked_get_release_by_id)
-        self.recording_patcher = patch('musicbrainzngs.get_recording_by_id',
-                                       side_effect=mocked_get_recording_by_id)
-        self.release_patcher.start()
-        self.recording_patcher.start()
-
     def tearDown(self):
-        self.recording_patcher.stop()
-        self.release_patcher.stop()
         self.teardown_beets()
 
     def test_one_mbid_one_album(self):
@@ -1796,76 +1956,6 @@ class ImportMusicBrainzIdTest(_common.TestCase, ImportHelper):
                          set([c.info.title for c in task.candidates]))
 
 
-# Helpers for ImportMusicBrainzIdTest.
-
-
-def mocked_get_release_by_id(id_, includes=[], release_status=[],
-                             release_type=[]):
-    """Mimic musicbrainzngs.get_release_by_id, accepting only a restricted list
-    of MB ids (ID_RELEASE_0, ID_RELEASE_1). The returned dict differs only in
-    the release title and artist name, so that ID_RELEASE_0 is a closer match
-    to the items created by ImportHelper._create_import_dir()."""
-    # Map IDs to (release title, artist), so the distances are different.
-    releases = {ImportMusicBrainzIdTest.ID_RELEASE_0: ('VALID_RELEASE_0',
-                                                       'TAG ARTIST'),
-                ImportMusicBrainzIdTest.ID_RELEASE_1: ('VALID_RELEASE_1',
-                                                       'DISTANT_MATCH')}
-
-    return {
-        'release': {
-            'title': releases[id_][0],
-            'id': id_,
-            'medium-list': [{
-                'track-list': [{
-                    'recording': {
-                        'title': 'foo',
-                        'id': 'bar',
-                        'length': 59,
-                    },
-                    'position': 9,
-                }],
-                'position': 5,
-            }],
-            'artist-credit': [{
-                'artist': {
-                    'name': releases[id_][1],
-                    'id': 'some-id',
-                },
-            }],
-            'release-group': {
-                'id': 'another-id',
-            }
-        }
-    }
-
-
-def mocked_get_recording_by_id(id_, includes=[], release_status=[],
-                               release_type=[]):
-    """Mimic musicbrainzngs.get_recording_by_id, accepting only a restricted
-    list of MB ids (ID_RECORDING_0, ID_RECORDING_1). The returned dict differs
-    only in the recording title and artist name, so that ID_RECORDING_0 is a
-    closer match to the items created by ImportHelper._create_import_dir()."""
-    # Map IDs to (recording title, artist), so the distances are different.
-    releases = {ImportMusicBrainzIdTest.ID_RECORDING_0: ('VALID_RECORDING_0',
-                                                         'TAG ARTIST'),
-                ImportMusicBrainzIdTest.ID_RECORDING_1: ('VALID_RECORDING_1',
-                                                         'DISTANT_MATCH')}
-
-    return {
-        'recording': {
-            'title': releases[id_][0],
-            'id': id_,
-            'length': 59,
-            'artist-credit': [{
-                'artist': {
-                    'name': releases[id_][1],
-                    'id': 'some-id',
-                },
-            }],
-        }
-    }
-
-
 def suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
 
