@@ -17,14 +17,15 @@
 
 from __future__ import division, absolute_import, print_function
 
+import unittest
 from mock import Mock
 
 from test import _common
-from test._common import unittest
 from beetsplug import lastgenre
 from beets import config
 
 from test.helper import TestHelper
+import six
 
 
 class LastGenrePluginTest(unittest.TestCase, TestHelper):
@@ -38,11 +39,11 @@ class LastGenrePluginTest(unittest.TestCase, TestHelper):
     def _setup_config(self, whitelist=False, canonical=False, count=1):
         config['lastgenre']['canonical'] = canonical
         config['lastgenre']['count'] = count
-        if isinstance(whitelist, (bool, basestring)):
+        if isinstance(whitelist, (bool, six.string_types)):
             # Filename, default, or disabled.
             config['lastgenre']['whitelist'] = whitelist
         self.plugin.setup()
-        if not isinstance(whitelist, (bool, basestring)):
+        if not isinstance(whitelist, (bool, six.string_types)):
             # Explicit list of genres.
             self.plugin.whitelist = whitelist
 
@@ -212,6 +213,18 @@ class LastGenrePluginTest(unittest.TestCase, TestHelper):
         self.assertEqual(res, (config['lastgenre']['fallback'].get(),
                          u'fallback'))
 
+    def test_sort_by_depth(self):
+        self._setup_config(canonical=True)
+        # Normal case.
+        tags = ('electronic', 'ambient', 'post-rock', 'downtempo')
+        res = self.plugin._sort_by_depth(tags)
+        self.assertEqual(
+            res, ['post-rock', 'downtempo', 'ambient', 'electronic'])
+        # Non-canonical tag ('chillout') present.
+        tags = ('electronic', 'ambient', 'chillout')
+        res = self.plugin._sort_by_depth(tags)
+        self.assertEqual(res, ['ambient', 'electronic'])
+
 
 def suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
