@@ -18,9 +18,10 @@
 from __future__ import division, absolute_import, print_function
 
 from test import _common
-from test._common import unittest
 from beets.autotag import mb
 from beets import config
+
+import unittest
 import mock
 
 
@@ -67,6 +68,7 @@ class MBAlbumInfoTest(_common.TestCase):
                 track = {
                     'recording': recording,
                     'position': i + 1,
+                    'number': 'A1',
                 }
                 if track_length:
                     # Track lengths are distinct from recording lengths.
@@ -181,6 +183,7 @@ class MBAlbumInfoTest(_common.TestCase):
         second_track_list = [{
             'recording': tracks[1],
             'position': '1',
+            'number': 'A1',
         }]
         release['medium-list'].append({
             'position': '2',
@@ -453,6 +456,7 @@ class MBLibraryTest(unittest.TestCase):
                                     'length': 42,
                                 },
                                 'position': 9,
+                                'number': 'A1',
                             }],
                             'position': 5,
                         }],
