@@ -20,18 +20,14 @@ from __future__ import division, absolute_import, print_function
 
 import os
 import shutil
-import tempfile
 import datetime
 import time
+import unittest
+from six import assertCountEqual
 
 from test import _common
-from test._common import unittest
-from beets.mediafile import MediaFile, MediaField, Image, \
-    MP3DescStorageStyle, StorageStyle, MP4StorageStyle, \
-    ASFStorageStyle, ImageType, CoverArtField
-from beets.library import Item
-from beets.plugins import BeetsPlugin
-from beets.util import bytestring_path
+from beets.mediafile import MediaFile, Image, \
+    ImageType, CoverArtField, UnreadableFileError
 
 
 class ArtTestMixin(object):
@@ -179,7 +175,11 @@ class ImageStructureTestMixin(ArtTestMixin):
 
 
 class ExtendedImageStructureTestMixin(ImageStructureTestMixin):
-    """Checks for additional attributes in the image structure."""
+    """Checks for additional attributes in the image structure.
+
+    Like the base `ImageStructureTestMixin`, per-format test classes
+    should include this mixin to add image-related tests.
+    """
 
     def assertExtendedImageAttributes(self, image, desc=None, type=None):  # noqa
         self.assertEqual(image.desc, desc)
@@ -268,7 +268,7 @@ class GenreListTestMixin(object):
 
     def test_read_genre_list(self):
         mediafile = self._mediafile_fixture('full')
-        self.assertItemsEqual(mediafile.genres, ['the genre'])
+        assertCountEqual(self, mediafile.genres, ['the genre'])
 
     def test_write_genre_list(self):
         mediafile = self._mediafile_fixture('empty')
@@ -276,7 +276,7 @@ class GenreListTestMixin(object):
         mediafile.save()
 
         mediafile = MediaFile(mediafile.path)
-        self.assertItemsEqual(mediafile.genres, [u'one', u'two'])
+        assertCountEqual(self, mediafile.genres, [u'one', u'two'])
 
     def test_write_genre_list_get_first(self):
         mediafile = self._mediafile_fixture('empty')
@@ -293,78 +293,28 @@ class GenreListTestMixin(object):
         mediafile.save()
 
         mediafile = MediaFile(mediafile.path)
-        self.assertItemsEqual(mediafile.genres, [u'the genre', u'another'])
-
-
-field_extension = MediaField(
-    MP3DescStorageStyle('customtag'),
-    MP4StorageStyle('----:com.apple.iTunes:customtag'),
-    StorageStyle('customtag'),
-    ASFStorageStyle('customtag'),
-)
-
-
-class ExtendedFieldTestMixin(object):
-
-    def test_extended_field_write(self):
-        plugin = BeetsPlugin()
-        plugin.add_media_field('customtag', field_extension)
-
-        mediafile = self._mediafile_fixture('empty')
-        mediafile.customtag = u'F#'
-        mediafile.save()
-
-        mediafile = MediaFile(mediafile.path)
-        self.assertEqual(mediafile.customtag, u'F#')
-        delattr(MediaFile, 'customtag')
-        Item._media_fields.remove('customtag')
-
-    def test_write_extended_tag_from_item(self):
-        plugin = BeetsPlugin()
-        plugin.add_media_field('customtag', field_extension)
-
-        mediafile = self._mediafile_fixture('empty')
-        self.assertIsNone(mediafile.customtag)
-
-        item = Item(path=mediafile.path, customtag=u'Gb')
-        item.write()
-        mediafile = MediaFile(mediafile.path)
-        self.assertEqual(mediafile.customtag, u'Gb')
-
-        delattr(MediaFile, 'customtag')
-        Item._media_fields.remove('customtag')
-
-    def test_read_flexible_attribute_from_file(self):
-        plugin = BeetsPlugin()
-        plugin.add_media_field('customtag', field_extension)
-
-        mediafile = self._mediafile_fixture('empty')
-        mediafile.update({'customtag': u'F#'})
-        mediafile.save()
-
-        item = Item.from_path(mediafile.path)
-        self.assertEqual(item['customtag'], u'F#')
-
-        delattr(MediaFile, 'customtag')
-        Item._media_fields.remove('customtag')
-
-    def test_invalid_descriptor(self):
-        with self.assertRaises(ValueError) as cm:
-            MediaFile.add_field('somekey', True)
-        self.assertIn(u'must be an instance of MediaField',
-                      unicode(cm.exception))
-
-    def test_overwrite_property(self):
-        with self.assertRaises(ValueError) as cm:
-            MediaFile.add_field('artist', MediaField())
-        self.assertIn(u'property "artist" already exists',
-                      unicode(cm.exception))
+        assertCountEqual(self, mediafile.genres, [u'the genre', u'another'])
 
 
 class ReadWriteTestBase(ArtTestMixin, GenreListTestMixin,
-                        ExtendedFieldTestMixin):
-    """Test writing and reading tags. Subclasses must set ``extension`` and
-    ``audio_properties``.
+                        _common.TempDirMixin):
+    """Test writing and reading tags. Subclasses must set ``extension``
+    and ``audio_properties``.
+
+    The basic tests for all audio formats encompass three files provided
+    in our `rsrc` folder: `full.*`, `empty.*`, and `unparseable.*`.
+    Respectively, they should contain a full slate of common fields
+    listed in `full_initial_tags` below; no fields contents at all; and
+    an unparseable release date field.
+
+    To add support for a new file format to MediaFile, add these three
+    files and then create a `ReadWriteTestBase` subclass by copying n'
+    pasting one of the existing subclasses below. You will want to
+    update the `format` field in that subclass, and you will probably
+    need to fiddle with the `bitrate` and other format-specific fields.
+
+    You can also add image tests (using an additional `image.*` fixture
+    file) by including one of the image-related mixins.
     """
 
     full_initial_tags = {
@@ -398,7 +348,10 @@ class ReadWriteTestBase(ArtTestMixin, GenreListTestMixin,
         'artist',
         'album',
         'genre',
+        'lyricist',
         'composer',
+        'composer_sort',
+        'arranger',
         'grouping',
         'year',
         'month',
@@ -421,6 +374,8 @@ class ReadWriteTestBase(ArtTestMixin, GenreListTestMixin,
         'rg_track_gain',
         'rg_album_peak',
         'rg_album_gain',
+        'r128_track_gain',
+        'r128_album_gain',
         'albumartist',
         'mb_albumartistid',
         'artist_sort',
@@ -447,11 +402,31 @@ class ReadWriteTestBase(ArtTestMixin, GenreListTestMixin,
     ]
 
     def setUp(self):
-        self.temp_dir = tempfile.mkdtemp()
+        self.create_temp_dir()
 
     def tearDown(self):
-        if os.path.isdir(self.temp_dir):
-            shutil.rmtree(self.temp_dir)
+        self.remove_temp_dir()
+
+    def test_read_nonexisting(self):
+        mediafile = self._mediafile_fixture('full')
+        os.remove(mediafile.path)
+        self.assertRaises(UnreadableFileError, MediaFile, mediafile.path)
+
+    def test_save_nonexisting(self):
+        mediafile = self._mediafile_fixture('full')
+        os.remove(mediafile.path)
+        try:
+            mediafile.save()
+        except UnreadableFileError:
+            pass
+
+    def test_delete_nonexisting(self):
+        mediafile = self._mediafile_fixture('full')
+        os.remove(mediafile.path)
+        try:
+            mediafile.delete()
+        except UnreadableFileError:
+            pass
 
     def test_read_audio_properties(self):
         mediafile = self._mediafile_fixture('full')
@@ -601,6 +576,9 @@ class ReadWriteTestBase(ArtTestMixin, GenreListTestMixin,
         self.assertEqual(mediafile.disctotal, None)
 
     def test_unparseable_date(self):
+        """The `unparseable.*` fixture should not crash but should return None
+        for all parts of the release date.
+        """
         mediafile = self._mediafile_fixture('unparseable')
 
         self.assertIsNone(mediafile.date)
@@ -679,7 +657,9 @@ class ReadWriteTestBase(ArtTestMixin, GenreListTestMixin,
             self.fail('\n  '.join(errors))
 
     def _mediafile_fixture(self, name):
-        name = bytestring_path(name + '.' + self.extension)
+        name = name + '.' + self.extension
+        if not isinstance(name, bytes):
+            name = name.encode('utf8')
         src = os.path.join(_common.RSRC, name)
         target = os.path.join(self.temp_dir, name)
         shutil.copy(src, target)
@@ -694,6 +674,9 @@ class ReadWriteTestBase(ArtTestMixin, GenreListTestMixin,
             if key.startswith('rg_'):
                 # ReplayGain is float
                 tags[key] = 1.0
+            elif key.startswith('r128_'):
+                # R128 is int
+                tags[key] = -1
             else:
                 tags[key] = 'value\u2010%s' % key
 
@@ -875,7 +858,7 @@ class FlacTest(ReadWriteTestBase, PartialTestMixin,
     extension = 'flac'
     audio_properties = {
         'length': 1.0,
-        'bitrate': 175120,
+        'bitrate': 108688,
         'format': u'FLAC',
         'samplerate': 44100,
         'bitdepth': 16,
@@ -932,6 +915,29 @@ class AIFFTest(ReadWriteTestBase, unittest.TestCase):
     }
 
 
+# Check whether we have a Mutagen version with DSF support. We can
+# remove this once we require a version that includes the feature.
+try:
+    import mutagen.dsf  # noqa
+except ImportError:
+    HAVE_DSF = False
+else:
+    HAVE_DSF = True
+
+
+@unittest.skipIf(not HAVE_DSF, "Mutagen does not have DSF support")
+class DSFTest(ReadWriteTestBase, unittest.TestCase):
+    extension = 'dsf'
+    audio_properties = {
+        'length': 0.01,
+        'bitrate': 11289600,
+        'format': u'DSD Stream File',
+        'samplerate': 5644800,
+        'bitdepth': 1,
+        'channels': 2,
+    }
+
+
 class MediaFieldTest(unittest.TestCase):
 
     def test_properties_from_fields(self):
@@ -949,7 +955,7 @@ class MediaFieldTest(unittest.TestCase):
     def test_known_fields(self):
         fields = list(ReadWriteTestBase.tag_fields)
         fields.extend(('encoder', 'images', 'genres', 'albumtype'))
-        self.assertItemsEqual(MediaFile.fields(), fields)
+        assertCountEqual(self, MediaFile.fields(), fields)
 
     def test_fields_in_readable_fields(self):
         readable = MediaFile.readable_fields()
