@@ -19,16 +19,16 @@ from __future__ import division, absolute_import, print_function
 
 import os
 import shutil
+import unittest
+import mutagen.id3
 
 from test import _common
-from test._common import unittest
-from test.helper import TestHelper
 
-from beets.util import bytestring_path
-import beets.mediafile
+from beets import mediafile
+import six
 
 
-_sc = beets.mediafile._safe_cast
+_sc = mediafile._safe_cast
 
 
 class EdgeTest(unittest.TestCase):
@@ -36,7 +36,7 @@ class EdgeTest(unittest.TestCase):
         # Some files have an ID3 frame that has a list with no elements.
         # This is very hard to produce, so this is just the first 8192
         # bytes of a file found "in the wild".
-        emptylist = beets.mediafile.MediaFile(
+        emptylist = mediafile.MediaFile(
             os.path.join(_common.RSRC, b'emptylist.mp3')
         )
         genre = emptylist.genre
@@ -45,7 +45,7 @@ class EdgeTest(unittest.TestCase):
     def test_release_time_with_space(self):
         # Ensures that release times delimited by spaces are ignored.
         # Amie Street produces such files.
-        space_time = beets.mediafile.MediaFile(
+        space_time = mediafile.MediaFile(
             os.path.join(_common.RSRC, b'space_time.mp3')
         )
         self.assertEqual(space_time.year, 2009)
@@ -55,7 +55,7 @@ class EdgeTest(unittest.TestCase):
     def test_release_time_with_t(self):
         # Ensures that release times delimited by Ts are ignored.
         # The iTunes Store produces such files.
-        t_time = beets.mediafile.MediaFile(
+        t_time = mediafile.MediaFile(
             os.path.join(_common.RSRC, b't_time.m4a')
         )
         self.assertEqual(t_time.year, 1987)
@@ -65,20 +65,20 @@ class EdgeTest(unittest.TestCase):
     def test_tempo_with_bpm(self):
         # Some files have a string like "128 BPM" in the tempo field
         # rather than just a number.
-        f = beets.mediafile.MediaFile(os.path.join(_common.RSRC, b'bpm.mp3'))
+        f = mediafile.MediaFile(os.path.join(_common.RSRC, b'bpm.mp3'))
         self.assertEqual(f.bpm, 128)
 
     def test_discc_alternate_field(self):
         # Different taggers use different vorbis comments to reflect
         # the disc and disc count fields: ensure that the alternative
         # style works.
-        f = beets.mediafile.MediaFile(os.path.join(_common.RSRC, b'discc.ogg'))
+        f = mediafile.MediaFile(os.path.join(_common.RSRC, b'discc.ogg'))
         self.assertEqual(f.disc, 4)
         self.assertEqual(f.disctotal, 5)
 
     def test_old_ape_version_bitrate(self):
         media_file = os.path.join(_common.RSRC, b'oldape.ape')
-        f = beets.mediafile.MediaFile(media_file)
+        f = mediafile.MediaFile(media_file)
         self.assertEqual(f.bitrate, 0)
 
     def test_only_magic_bytes_jpeg(self):
@@ -86,17 +86,17 @@ class EdgeTest(unittest.TestCase):
         # such aren't recognized by imghdr. Ensure that this still works thanks
         # to our own follow up mimetype detection based on
         # https://github.com/file/file/blob/master/magic/Magdir/jpeg#L12
-        f = open(os.path.join(_common.RSRC, b'only-magic-bytes.jpg'), 'rb')
-        jpg_data = f.read()
+        magic_bytes_file = os.path.join(_common.RSRC, b'only-magic-bytes.jpg')
+        with open(magic_bytes_file, 'rb') as f:
+            jpg_data = f.read()
         self.assertEqual(
-            beets.mediafile._image_mime_type(jpg_data),
-            'image/jpeg')
+            mediafile._imghdr_what_wrapper(jpg_data), 'jpeg')
 
     def test_soundcheck_non_ascii(self):
         # Make sure we don't crash when the iTunes SoundCheck field contains
         # non-ASCII binary data.
-        f = beets.mediafile.MediaFile(os.path.join(_common.RSRC,
-                                                   b'soundcheck-nonascii.m4a'))
+        f = mediafile.MediaFile(os.path.join(_common.RSRC,
+                                             b'soundcheck-nonascii.m4a'))
         self.assertEqual(f.rg_track_gain, 0.0)
 
 
@@ -105,6 +105,9 @@ class InvalidValueToleranceTest(unittest.TestCase):
     def test_safe_cast_string_to_int(self):
         self.assertEqual(_sc(int, u'something'), 0)
 
+    def test_safe_cast_string_to_int_with_no_numbers(self):
+        self.assertEqual(_sc(int, u'-'), 0)
+
     def test_safe_cast_int_string_to_int(self):
         self.assertEqual(_sc(int, u'20'), 20)
 
@@ -127,8 +130,8 @@ class InvalidValueToleranceTest(unittest.TestCase):
         self.assertAlmostEqual(_sc(float, u'-1.234'), -1.234)
 
     def test_safe_cast_special_chars_to_unicode(self):
-        us = _sc(unicode, 'caf\xc3\xa9')
-        self.assertTrue(isinstance(us, unicode))
+        us = _sc(six.text_type, 'caf\xc3\xa9')
+        self.assertTrue(isinstance(us, six.text_type))
         self.assertTrue(us.startswith(u'caf'))
 
     def test_safe_cast_float_with_no_numbers(self):
@@ -144,7 +147,7 @@ class InvalidValueToleranceTest(unittest.TestCase):
         self.assertEqual(v, 1.0)
 
 
-class SafetyTest(unittest.TestCase, TestHelper):
+class SafetyTest(unittest.TestCase, _common.TempDirMixin):
     def setUp(self):
         self.create_temp_dir()
 
@@ -156,44 +159,44 @@ class SafetyTest(unittest.TestCase, TestHelper):
         with open(fn, 'w') as f:
             f.write(data)
         try:
-            self.assertRaises(exc, beets.mediafile.MediaFile, fn)
+            self.assertRaises(exc, mediafile.MediaFile, fn)
         finally:
             os.unlink(fn)  # delete the temporary file
 
     def test_corrupt_mp3_raises_unreadablefileerror(self):
         # Make sure we catch Mutagen reading errors appropriately.
-        self._exccheck(b'corrupt.mp3', beets.mediafile.UnreadableFileError)
+        self._exccheck(b'corrupt.mp3', mediafile.UnreadableFileError)
 
     def test_corrupt_mp4_raises_unreadablefileerror(self):
-        self._exccheck(b'corrupt.m4a', beets.mediafile.UnreadableFileError)
+        self._exccheck(b'corrupt.m4a', mediafile.UnreadableFileError)
 
     def test_corrupt_flac_raises_unreadablefileerror(self):
-        self._exccheck(b'corrupt.flac', beets.mediafile.UnreadableFileError)
+        self._exccheck(b'corrupt.flac', mediafile.UnreadableFileError)
 
     def test_corrupt_ogg_raises_unreadablefileerror(self):
-        self._exccheck(b'corrupt.ogg', beets.mediafile.UnreadableFileError)
+        self._exccheck(b'corrupt.ogg', mediafile.UnreadableFileError)
 
     def test_invalid_ogg_header_raises_unreadablefileerror(self):
-        self._exccheck(b'corrupt.ogg', beets.mediafile.UnreadableFileError,
+        self._exccheck(b'corrupt.ogg', mediafile.UnreadableFileError,
                        'OggS\x01vorbis')
 
     def test_corrupt_monkeys_raises_unreadablefileerror(self):
-        self._exccheck(b'corrupt.ape', beets.mediafile.UnreadableFileError)
+        self._exccheck(b'corrupt.ape', mediafile.UnreadableFileError)
 
     def test_invalid_extension_raises_filetypeerror(self):
-        self._exccheck(b'something.unknown', beets.mediafile.FileTypeError)
+        self._exccheck(b'something.unknown', mediafile.FileTypeError)
 
     def test_magic_xml_raises_unreadablefileerror(self):
-        self._exccheck(b'nothing.xml', beets.mediafile.UnreadableFileError,
+        self._exccheck(b'nothing.xml', mediafile.UnreadableFileError,
                        "ftyp")
 
-    @unittest.skipIf(not hasattr(os, 'symlink'), u'platform lacks symlink')
+    @unittest.skipUnless(_common.HAVE_SYMLINK, u'platform lacks symlink')
     def test_broken_symlink(self):
         fn = os.path.join(_common.RSRC, b'brokenlink')
         os.symlink('does_not_exist', fn)
         try:
-            self.assertRaises(IOError,
-                              beets.mediafile.MediaFile, fn)
+            self.assertRaises(mediafile.UnreadableFileError,
+                              mediafile.MediaFile, fn)
         finally:
             os.unlink(fn)
 
@@ -204,19 +207,19 @@ class SideEffectsTest(unittest.TestCase):
 
     def test_opening_tagless_file_leaves_untouched(self):
         old_mtime = os.stat(self.empty).st_mtime
-        beets.mediafile.MediaFile(self.empty)
+        mediafile.MediaFile(self.empty)
         new_mtime = os.stat(self.empty).st_mtime
         self.assertEqual(old_mtime, new_mtime)
 
 
-class MP4EncodingTest(unittest.TestCase, TestHelper):
+class MP4EncodingTest(unittest.TestCase, _common.TempDirMixin):
     def setUp(self):
         self.create_temp_dir()
         src = os.path.join(_common.RSRC, b'full.m4a')
         self.path = os.path.join(self.temp_dir, b'test.m4a')
         shutil.copy(src, self.path)
 
-        self.mf = beets.mediafile.MediaFile(self.path)
+        self.mf = mediafile.MediaFile(self.path)
 
     def tearDown(self):
         self.remove_temp_dir()
@@ -224,18 +227,18 @@ class MP4EncodingTest(unittest.TestCase, TestHelper):
     def test_unicode_label_in_m4a(self):
         self.mf.label = u'foo\xe8bar'
         self.mf.save()
-        new_mf = beets.mediafile.MediaFile(self.path)
+        new_mf = mediafile.MediaFile(self.path)
         self.assertEqual(new_mf.label, u'foo\xe8bar')
 
 
-class MP3EncodingTest(unittest.TestCase, TestHelper):
+class MP3EncodingTest(unittest.TestCase, _common.TempDirMixin):
     def setUp(self):
         self.create_temp_dir()
         src = os.path.join(_common.RSRC, b'full.mp3')
         self.path = os.path.join(self.temp_dir, b'test.mp3')
         shutil.copy(src, self.path)
 
-        self.mf = beets.mediafile.MediaFile(self.path)
+        self.mf = mediafile.MediaFile(self.path)
 
     def test_comment_with_latin1_encoding(self):
         # Set up the test file with a Latin1-encoded COMM frame. The encoding
@@ -248,7 +251,7 @@ class MP3EncodingTest(unittest.TestCase, TestHelper):
         self.mf.save()
 
 
-class ZeroLengthMediaFile(beets.mediafile.MediaFile):
+class ZeroLengthMediaFile(mediafile.MediaFile):
     @property
     def length(self):
         return 0.0
@@ -269,7 +272,7 @@ class TypeTest(unittest.TestCase):
     def setUp(self):
         super(TypeTest, self).setUp()
         path = os.path.join(_common.RSRC, b'full.mp3')
-        self.mf = beets.mediafile.MediaFile(path)
+        self.mf = mediafile.MediaFile(path)
 
     def test_year_integer_in_string(self):
         self.mf.year = u'2009'
@@ -301,45 +304,45 @@ class TypeTest(unittest.TestCase):
 
 class SoundCheckTest(unittest.TestCase):
     def test_round_trip(self):
-        data = beets.mediafile._sc_encode(1.0, 1.0)
-        gain, peak = beets.mediafile._sc_decode(data)
+        data = mediafile._sc_encode(1.0, 1.0)
+        gain, peak = mediafile._sc_decode(data)
         self.assertEqual(gain, 1.0)
         self.assertEqual(peak, 1.0)
 
     def test_decode_zero(self):
         data = b' 80000000 80000000 00000000 00000000 00000000 00000000 ' \
                b'00000000 00000000 00000000 00000000'
-        gain, peak = beets.mediafile._sc_decode(data)
+        gain, peak = mediafile._sc_decode(data)
         self.assertEqual(gain, 0.0)
         self.assertEqual(peak, 0.0)
 
     def test_malformatted(self):
-        gain, peak = beets.mediafile._sc_decode(b'foo')
+        gain, peak = mediafile._sc_decode(b'foo')
         self.assertEqual(gain, 0.0)
         self.assertEqual(peak, 0.0)
 
     def test_special_characters(self):
-        gain, peak = beets.mediafile._sc_decode(u'caf\xe9'.encode('utf8'))
+        gain, peak = mediafile._sc_decode(u'caf\xe9'.encode('utf-8'))
         self.assertEqual(gain, 0.0)
         self.assertEqual(peak, 0.0)
 
     def test_decode_handles_unicode(self):
         # Most of the time, we expect to decode the raw bytes. But some formats
         # might give us text strings, which we need to handle.
-        gain, peak = beets.mediafile._sc_decode(u'caf\xe9')
+        gain, peak = mediafile._sc_decode(u'caf\xe9')
         self.assertEqual(gain, 0.0)
         self.assertEqual(peak, 0.0)
 
 
-class ID3v23Test(unittest.TestCase, TestHelper):
-    def _make_test(self, ext='mp3', id3v23=False):
+class ID3v23Test(unittest.TestCase, _common.TempDirMixin):
+    def _make_test(self, ext=b'mp3', id3v23=False):
         self.create_temp_dir()
         src = os.path.join(_common.RSRC,
-                           bytestring_path('full.{0}'.format(ext)))
+                           b'full.' + ext)
         self.path = os.path.join(self.temp_dir,
-                                 bytestring_path('test.{0}'.format(ext)))
+                                 b'test.' + ext)
         shutil.copy(src, self.path)
-        return beets.mediafile.MediaFile(self.path, id3v23=id3v23)
+        return mediafile.MediaFile(self.path, id3v23=id3v23)
 
     def _delete_test(self):
         self.remove_temp_dir()
@@ -350,7 +353,7 @@ class ID3v23Test(unittest.TestCase, TestHelper):
             mf.year = 2013
             mf.save()
             frame = mf.mgfile['TDRC']
-            self.assertTrue('2013' in unicode(frame))
+            self.assertTrue('2013' in six.text_type(frame))
             self.assertTrue('TYER' not in mf.mgfile)
         finally:
             self._delete_test()
@@ -361,43 +364,43 @@ class ID3v23Test(unittest.TestCase, TestHelper):
             mf.year = 2013
             mf.save()
             frame = mf.mgfile['TYER']
-            self.assertTrue('2013' in unicode(frame))
+            self.assertTrue('2013' in six.text_type(frame))
             self.assertTrue('TDRC' not in mf.mgfile)
         finally:
             self._delete_test()
 
     def test_v23_on_non_mp3_is_noop(self):
-        mf = self._make_test('m4a', id3v23=True)
+        mf = self._make_test(b'm4a', id3v23=True)
         try:
             mf.year = 2013
             mf.save()
         finally:
             self._delete_test()
 
-    def test_v24_image_encoding(self):
-        mf = self._make_test(id3v23=False)
-        try:
-            mf.images = [beets.mediafile.Image(b'test data')]
-            mf.save()
-            frame = mf.mgfile.tags.getall('APIC')[0]
-            self.assertEqual(frame.encoding, 3)
-        finally:
-            self._delete_test()
+    def test_image_encoding(self):
+        """For compatibility with OS X/iTunes.
 
-    @unittest.skip("a bug, see #899")
-    def test_v23_image_encoding(self):
-        """For compatibility with OS X/iTunes (and strict adherence to
-        the standard), ID3v2.3 tags need to use an inferior text
-        encoding: UTF-8 is not supported.
+        See https://github.com/beetbox/beets/issues/899#issuecomment-62437773
         """
-        mf = self._make_test(id3v23=True)
-        try:
-            mf.images = [beets.mediafile.Image(b'test data')]
-            mf.save()
-            frame = mf.mgfile.tags.getall('APIC')[0]
-            self.assertEqual(frame.encoding, 1)
-        finally:
-            self._delete_test()
+
+        for v23 in [True, False]:
+            mf = self._make_test(id3v23=v23)
+            try:
+                mf.images = [
+                    mediafile.Image(b'data', desc=u""),
+                    mediafile.Image(b'data', desc=u"foo"),
+                    mediafile.Image(b'data', desc=u"\u0185"),
+                ]
+                mf.save()
+                apic_frames = mf.mgfile.tags.getall('APIC')
+                encodings = dict([(f.desc, f.encoding) for f in apic_frames])
+                self.assertEqual(encodings, {
+                    u"": mutagen.id3.Encoding.LATIN1,
+                    u"foo": mutagen.id3.Encoding.LATIN1,
+                    u"\u0185": mutagen.id3.Encoding.UTF16,
+                })
+            finally:
+                self._delete_test()
 
 
 def suite():
