@@ -16,7 +16,9 @@
 
 from __future__ import division, absolute_import, print_function
 
-from test._common import unittest
+import unittest
+import six
+
 from test.helper import TestHelper, has_program
 
 from beets import config
@@ -50,19 +52,19 @@ class ReplayGainCliTestBase(TestHelper):
 
         try:
             self.load_plugins('replaygain')
-        except:
+        except Exception:
             import sys
             # store exception info so an error in teardown does not swallow it
             exc_info = sys.exc_info()
             try:
                 self.teardown_beets()
                 self.unload_plugins()
-            except:
+            except Exception:
                 # if load_plugins() failed then setup is incomplete and
                 # teardown operations may fail. In particular # {Item,Album}
                 # may not have the _original_types attribute in unload_plugins
                 pass
-            raise exc_info[1], None, exc_info[2]
+            six.reraise(exc_info[1], None, exc_info[2])
 
         album = self.add_album_fixture(2)
         for item in album.items():
